/* Colorpicker -- Picks a color for a directory entry
   Copyright (C) 2002 Aaron Bentley

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
class ColorPicker
{
public:
   virtual Gdk_Color operator() (EntryRect const &er)
   {
      Gdk_Color color;
      er.setColor(color);
      return color;
   }
   virtual ~ColorPicker()
   {}
};

class GidColor: public ColorPicker
{
private:
   std::map<gid_t, Gdk_Color> gidcolor;

public:
   GidColor();

   virtual Gdk_Color operator() (EntryRect const &er)
   {
      std::map<gid_t, Gdk_Color>::const_iterator color;
      color=gidcolor.find(er.get_EnTree().getGid());
      if (color!=gidcolor.end())
      {
	 return color->second;
      }
      else
      {
	 Gdk_Color white;
	 white.set_rgb(65535,65535,65535);
	 return white;
      }
   }
};

class AgeColor: public ColorPicker
{
   private:
   time_t curtime;
   time_t oldest;

   public:
   AgeColor()
      :
   curtime(time(NULL))
   {
      oldest=60*60*24*365; 
   }
   virtual Gdk_Color operator() (EntryRect const &er)
   {
      //const long yearsec=60*60*24*365;
      Gdk_Color agecolor;
      float age=curtime-er.get_EnTree().getATime();
      gint red=static_cast<gint>(age/oldest*65535);

      //gint green=er.get_EnTree().getMTime();
      //green=green%65535;
      agecolor.set_rgb(red, 65535-red, 0);
      return agecolor;
   }
};
