#include "drawerdep.h"
#include <memory>
#include <gtk--/button.h>
#include "colorpickerdep.h"
#include "dutree.h"
#include <cmath>

/* drawer -- class that draws rectangles representing files 
   Copyright (C) 2002 Aaron Bentley

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

Drawer::Drawer(Gdk_Window const awindow)
      :
colormap(Gdk_Colormap::get_system()), window(awindow)
{
   context.create(window);
}

void Drawer::operator()(EntryRect const &cur, Gdk_Color curcolor)
{
   colormap.alloc(curcolor);
   context.set_foreground(curcolor);
   window.draw_rectangle(context, true, static_cast<int>(cur.left), 
      static_cast<int>(cur.top), static_cast<int>(cur.width+1), 
      static_cast<int>(cur.height));
}

PrettyDrawer::PrettyDrawer(Gdk_Window const awindow)
   : Drawer(awindow)
{}

void PrettyDrawer::operator()(EntryRect const &cur, Gdk_Color curcolor)
{
   float left=cur.left;
   float top=cur.top;
   float width=cur.width;
   float height=cur.height;
   float brightness=1;
   for (; width>0 && height >0; left++, width-=2, top++, height-=2, brightness*=.985)
   {
      fg_brightness(curcolor, brightness);
      window.draw_rectangle(context, true, static_cast<int>(left), 
	 static_cast<int>(top), static_cast<int>(width+1), 
	 static_cast<int>(height));
   }
}

void PrettyDrawer::brightness(Gdk_Color &curcolor, float amount) const
{
   curcolor.set_red(static_cast<int>(curcolor.get_red()*amount));
   curcolor.set_green(static_cast<int>(curcolor.get_green()*amount));
   curcolor.set_blue(static_cast<int>(curcolor.get_blue()*amount));
}

void PrettyDrawer::fg_brightness(Gdk_Color curcolor, float amount)
{
   brightness(curcolor, amount);
   colormap.alloc(curcolor);
   context.set_foreground(curcolor);
}

DarkFirst::DarkFirst(Gdk_Window const awindow)
   : PrettyDrawer(awindow)
{}

void DarkFirst::operator()(EntryRect const &cur, Gdk_Color curcolor)
{
   float left=cur.left;
   float top=cur.top;
   float width=cur.width;
   float height=cur.height;
   float brightness=1;
   for (; width>0 && height >0; left++, width-=2, top++, height-=2, brightness*=.9)
   {
      brightness=width*height/cur.width/cur.height;
      brightness=1-(sqrtf(brightness)*.70);
      fg_brightness(curcolor, brightness);
      window.draw_rectangle(context, true, static_cast<int>(left),
	 static_cast<int>(top), static_cast<int>(width+1), 
	 static_cast<int>(height));
   }
}
