/* Typelist -- lists of types for template programming
   Copyright (C) 2002 Aaron Bentley

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
/**
 * \brief Contains general metaprogramming functions, objects, etc.
 *
 * Metaprogramming is code that is executed at compile time, and is based on
 * templates.
 * \par Conditionals
 * Conditionals are accomplished with template specialization.
 *
 * \par Loops
 * Loops are accomplished using recursion and conditionals
 *
 * \par Functions
 * Functions are template classes that define a Ret (for "return value")
 * member that is a typedef or an enum; that definition is the output.
 */
namespace meta
{
   ///The Nil class is the empty typelist.  It is used to terminate other
   //typelists
   class Nil{};

   ///This class is a typelist
   template <typename H, typename T=Nil>
   class Typelist 
   {
      public:
      ///The actual type available
      typedef H Head;

      ///The rest of the list
      typedef T Tail;
   };

   /**
    * \brief Function to produce a TypeList from the <=7 types supplied
    * as arguments
    */
   template <typename T1=Nil, typename T2=Nil, typename T3=Nil,
      typename T4=Nil, typename T5=Nil, typename T6=Nil, typename T7=Nil>
   class List
   {
		public:
      ///The Typelist return value
      typedef Typelist<T1, typename List<T2, T3, T4, T5, T6, T7>::Ret > Ret;
   };

   /**
    * \brief Function to produce a TypeList from the <=7 types supplied
    * as arguments-- handler for end-of input
    */
   template<>
   class List<Nil, Nil, Nil, Nil, Nil, Nil, Nil>
   {
		public:
      ///The typelist return value;
      typedef Nil Ret;
   };
}
